/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "Game.h"

#include <assert.h>

Game::Game(Uint8 sz, const string &black, const string &white) : board(sz), size(sz) {
  assert((black != "") || (white != ""));
  assert(sz < 25);
  
  if (black != "")
    programB = new GTP(black);
  else
    programB = NULL;
    
  if (white != "") {
    if (white == black)
      programW = programB;
    else
      programW = new GTP(white);
  } else
    programW = NULL;

  if (programB != NULL) programB->boardsize(sz);
  if ((programW != NULL) && (programW != programB)) programW->boardsize(sz);
}

Game::~Game() {
  if (programB != NULL) programB->quit();
  if (programW != NULL) programW->quit();
  delete programB;
  delete programW;
}

bool Game::blackM(const Vertex &v) {
  GTP *program = (programB != NULL) ? programB : programW;
  if (!program->cmd_blackLegal(v)) return false;
  if (programB != NULL) programB->moveBlack(v);
  if ((programW != NULL) && (programB != programW)) programW->moveBlack(v);
  updateBoard(v);
  last = v;
  return true;
}
void Game::blackP() {
  last = Vertex();
}
void Game::blackC() {
  assert(programB != NULL);
  int c = programB->cmd_genmoveBlack();
  
  Vertex v;
  while (!programB->res_genmove(c, v));
  
  // the returned vertex is invalid in case of a pass
  if (v.isValid()) {
    if ((programW != NULL) && (programW != programB)) programW->moveBlack(v);
    updateBoard(v);
  } 
  last = v;
}
bool Game::whiteM(const Vertex &v) {
  GTP *program = (programB != NULL) ? programB : programW;
  if (!program->cmd_whiteLegal(v)) return false;
  if (programB != NULL) programB->moveWhite(v);
  if ((programW != NULL) && (programB != programW)) programW->moveWhite(v);
  updateBoard(v);
  last = v;
  return true;
}
void Game::whiteC() {
  assert(programW != NULL);
  
  int c = programW->cmd_genmoveWhite();
  
  Vertex v;
  while (!programW->res_genmove(c, v));
  
  if (v.isValid()) {
    if ((programB != NULL) && (programW != programB)) programB->moveWhite(v);
    updateBoard(v);
  }
  
  last = v;
}
void Game::whiteP() {
  last = Vertex();
}

void Game::updateBoard(const Vertex  &v) {
  GTP *program = (programB != NULL) ? programB : programW;
  Uint8 nColor = program->cmd_getVertex(v);
  
  if (nColor != board.getVertex(v)) {
    board.setVertex(v, nColor);
    if (v.row() > 0) updateBoard(Vertex(v.row()-1, v.col()));
    if (v.col() > 0) updateBoard(Vertex(v.row(), v.col()-1));
    if (v.row() < size-1) updateBoard(Vertex(v.row()+1, v.col()));
    if (v.col() < size-1) updateBoard(Vertex(v.row(), v.col()+1));
  }
}

void Game::newGame(const Uint8 boardsize) {
  if (programB != NULL) programB->boardsize(boardsize);
  if (programW != NULL) programW->boardsize(boardsize);
  board.clearBoard();
}

void Game::loadSGF(string filename) {
  if (programB != NULL) programB->loadSGF(filename);
  if (programW != NULL) programW->loadSGF(filename);
  GTP *program = (programB != NULL) ? programB : programW;
  
  // load the whole board now
  // FIXME: ask gngo for the actual boardsize
  
  for (int y = 0; y < 19; y++)
    for (int x = 0; x < 19; x++)
      board.setVertex(Vertex(y, x), program->cmd_getVertex(Vertex(y, x)));
}