/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "Options.h"

#include <iostream>
#include <fstream>
#include <strstream>

/**
 * default values
 */
Options::Options() {
  boardsize = 19;
  blackplayer = "";
  whiteplayer = "";
  handicap = 0;
  
  ruleset = japanese;
}

Options::~Options() {}

void Options::load() {
  string line(getenv("HOME")), var, val;
  line += "/.SDLGo";
  ifstream in(line.c_str());
  int tmp;
  unsigned int p;
  
  while (getline(in, line)) {
    p = line.find(" ");
    var = line.substr(0, p);
    if (line.length() > p)
      val = line.substr(p + 1);
    else
      val = "";
    if        (var == "boardsize") { istrstream(val.c_str()) >> tmp; boardsize = tmp;
    } else if (var == "black")     { blackplayer = val;
    } else if (var == "white")     { whiteplayer = val;
    } else if (var == "graphics")  { graphicsFile = val;
    } else if (var == "handicap")  { istrstream(val.c_str()) >> tmp; handicap = tmp;
    } else if (var == "ruleset") {
      if        (val == "japanese") { ruleset = japanese;
      } else if (val == "chinese")  { ruleset = chinese;
      }
    }
  }
}
void Options::save() {
  string line(getenv("HOME"));
  line += "/.SDLGo";
  ofstream out(line.c_str());
  
  out << "boardsize " << boardsize << endl;
  out << "black " << blackplayer << endl;
  out << "white " << whiteplayer << endl;
  out << "ruleset ";
  switch (ruleset) {
    case japanese: out << "japanese" << endl;
    case chinese: out << "chinese" << endl;
    default: out << "unknown" << endl;
  }
  out << "graphics " << graphicsFile << endl;
  out << "handicap " << handicap << endl;
}
    
Uint8 Options::getBoardsize() { return boardsize; }
Uint8 Options::getHandicap() { return handicap; }

string& Options::getBlackplayer() { return blackplayer; }
string& Options::getWhiteplayer() { return whiteplayer; }
string& Options::getGraphicsFile() { return graphicsFile; }

rulesets Options::getRulset() { return ruleset; }